@@ -1,10 +1,11 @@
 from collections import defaultdict
 from copy import deepcopy
 from itertools import chain
-from typing import Callable, Dict, FrozenSet, Generic, Iterable, Iterator, List, Optional, Type, TypeVar
+from typing import Callable, Dict, FrozenSet, Generic, Iterable, Iterator, List, Optional, Type, TypeVar, Union
 
 from tdm.abstract.datamodel import AbstractDirective, AbstractFact, AbstractTalismanDocument, AbstractTreeDocumentContent, FactType
 from tdm.abstract.datamodel.document import DocumentMetadata
+from tdm.datamodel.fact import ConceptFact, PropertyFact, RelationFact, ValueFact
 from .fact import PropertyLinkValue
 
 _TalismanDocument = TypeVar('_TalismanDocument', bound='TalismanDocument')
@@ -160,3 +161,16 @@ class TalismanDocument(AbstractTalismanDocument[_Content], Generic[_Content]):
 
     def __hash__(self):
         return hash((self._doc_id, self._content, self._metadata, self._facts, self._directives))
+
+    def get_facts_related2concept(self, concept_fact: ConceptFact, type_: Type[_Fact],
+                                  filter_: Callable[[_Fact], bool] = lambda _: True) -> Iterator[Union[ConceptFact, ValueFact]]:
+        for fact in chain(self.filter_facts(RelationFact), self.filter_facts(PropertyFact)):
+            related_fact = None
+            if fact.value.from_fact == concept_fact:
+                related_fact = fact.value.to_fact
+
+            elif fact.value.to_fact == concept_fact:
+                related_fact = fact.value.from_fact
+
+            if isinstance(related_fact, type_) and filter_(related_fact):
+                yield related_fact
